//###############################################################################
//#
//# File:         ImapC.java
//# RCS:          $Header: /u/ejk/var/cvs/courses/cs161/login-solution/ImapC.java,v 1.2 2005/12/02 13:11:18 ejk Exp $
//# Description:  A multi-server IMAP client
//# Author:       Eric Koskinen
//# Created:      Wed Sep 14 12:48:49 2005
//# Modified:     Wed Sep 14 12:48:49 2005 (Eric Koskinen) ejk@cs.brown.edu
//# Language:     Java
//# Package:      N/A
//# Status:       Experimental (Do Not Distribute)
//#
//# (C/) Copyright 2005, Eric J Koskinen, all rights reserved.
//#
//###############################################################################

import java.io.*;
import java.util.*;
import java.nio.*;
import java.nio.channels.*;

public class ImapC {

    public static ArrayList<Account> accounts;

    // ****************************************************************
    //
    // ImapC's main()
    //
    // ****************************************************************

    public static void main (String[] args) throws IOException {

	if(args.length != 1) usage();

	init(args[0]);

    }

    // ****************************************************************
    //
    // usage: print the usage and exit
    //
    // ****************************************************************

    private static void usage () {

	System.err.println("usage: java ImapC <config>");
	
	System.exit(1);
    }

    // ****************************************************************
    //
    // init: load config, parse hosts/commands, and fire up threads
    //
    // ****************************************************************

    private static void init(String config_fn) throws FileNotFoundException, IOException {

	// read accounts from config (non-NIO :-/)
	FileReader fr;
	try {
	    fr = new FileReader(config_fn);
	} catch (FileNotFoundException e) {
	    System.err.println("no such file: "+config_fn);
	    return;
	}
	BufferedReader br = new BufferedReader( fr );
	ArrayList<String> cLines = new ArrayList<String>( 1024 );
	accounts = new ArrayList<Account>();

	// read lines of the file, and instantiate a new Account
	// for each config block
	String line = null;
	int account_id = 0;
	do {
	    line = br.readLine();
	    // keep track of this line
	    if (line.compareTo("") != 0 && line.charAt(0) != '#')
		cLines.add( line );
	    // see if we've got a completed stanza
	    if (!cLines.isEmpty() && (line.compareTo("") == 0 || !br.ready())) {
		// empty line means account is ready to be made
		Account a = init_create_account(cLines.get(0).toString(),account_id++);
		// queue the commands
		if(cLines.size()>1)
		    a.queueCommands( cLines.subList(1,cLines.size()) );
		accounts.add( a );
		cLines.clear();
	    }
	} while (br.ready());
	br.close();
	fr.close();

	System.err.println("---+ ImapC +----------------------------");

	/* +++ ONE THREAD PER ACCOUNT +++ */

	// fire up the threads
	for (Account a : accounts)
	    a.start();

	// join back up with them
	for (Account a : accounts) {
	    try {
		a.join();
	    } catch (InterruptedException e) {
		System.err.println("Thread interrupted: " + e.getMessage());
	    }
	}

	System.err.println("----------------------------------------");
	System.err.println("Completed  "+accounts.size()+ " clients.");
	
    }

    // ****************************************************************
    //
    // init_create_account: parse the hostname/port, and return a new
    //   Account
    //
    // ****************************************************************

    private static Account init_create_account(String parse_me, int id)
      throws IOException {
	Account act;
	int spacer  = parse_me.indexOf(' ');
	String host = parse_me.substring(0,spacer);
	String port = parse_me.substring(spacer+1,parse_me.length());
	Integer i   = new Integer(id);
	act = new Account("c"+i.toString()+"="+host+":"+port);
	act.init(host,port);
	return act;
    }

}
